#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 21/10/2002
#################################################################################
#
# Home - Home page component for Ethereal-Realms 
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date processing
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table creation
use Ethereal::Template;						# Template handling


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $table;							# Table handler
my $tmpl;							# Template handle

my $expire;							# Expired homepages
my $upload;							# Uploaded file

my %home;							# Home page hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();
	$table    = new Ethereal::Table();

	# Retrieve upload handle
	$upload = $cgi->upload('FUPLOAD');			# Retreive upload handle

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Retreive hash
	$database->GetHashHome(\%home);

	# Create instance
	$tmpl = new Ethereal::Template(\%home);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Should system be active
	if ($home{'SetActive'})
	{
		# Login handle
		$login = new Ethereal::Login($database, $cgi, $param);

		# Verify login and continue with system
		if ($login->GetVerificationNormal())
		{
			# Expired or not?
			($expire) = $database->DataGetPuppeteerExpire($param->{'USER'});

			# Account still valid
			if ($expire > time)
			{
				# Make sure Directory is sane 
				if (DirHandler($database, $cgi, $param, $table, $tmpl, \%home))
				{
					# Display title
					$tmpl->ShowMain($home{'TagTitle'});

					unless (defined($param->{'LISTING'}))
					{
						# Handling of folder
						HomeHandler($database, $cgi, $param, $table, $tmpl, \%home, $upload);
					}
					else
					{
						# Directory listing
						DirListing($database, $cgi, $param, $table, $tmpl, \%home);
					}
				}
			}
			else
			{
				# Display disabled message
				$table->MakeTop();
				$table->MakeSingle($home{'MsgExpire'});
				$table->MakeBottom();
			}
		}
	}
	else
	{
		# Display disabled message
		$table->MakeTop();
		$table->MakeSingle($home{'MsgDisabled'});
		$table->MakeBottom();
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Directory Handler 
#
# This will make sure that certain components of Ethereals Home page system,
# primarily the home page directory is in order.   This means creation, prompt
# or whatever is needed to get it going. 

sub DirHandler 
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handler 
	my $tmpl     = shift;					# Template handle
	my $home     = shift;					# Home page hash handle 
	my $message  = shift;					# Error message

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $dir;						# Directory

	my $header;						# Header file
	my $readme;						# Readme file


	#####################
	# Program area

	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

 	# Determin status 
	# Directory defined 
	if (defined($dir))
	{
		# Ensure existence
		unless (-d "$home->{SetRoot}$dir")
		{
			# Create and verify
			unless (mkdir("$home->{SetRoot}$dir", 0775))
			{
				# Display error message
				$table->MakeTop();
				$table->MakeSingle($home->{'MsgFail'});
				$table->MakeBlank();
				$table->MakeSingle("<CENTER>$!</CENTER>");
				$table->MakeBottom();

				# Reset Database
				$data->do("UPDATE Puppeteer 
					SET   PuppeteerHome=NULL 
					WHERE PuppeteerLogin=?", {}, 

				 $param->{'USER'});	
				
				# Return False
				return 0;
			} 
		}

		# Retreive header and readme
		($header) = $database->DocumentGetHomeHeader();
		($readme) = $database->DocumentGetHomeReadme();

		# Write contents
		FileWrite("$home->{SetRoot}$dir/$home->{SetHEADER}", $header);
		FileWrite("$home->{SetRoot}$dir/$home->{SetREADME}", $readme);

		# Return true
		return 1;
	}

	# Directory creation
	elsif ((defined($param->{'DCREATE'})) && ($param->{'DCREATE'} !~ /\W+/g))
	{
		# Parameter handling
		$dir = $param->{'DCREATE'};
		delete $param->{'DCREATE'};


		# Ensure existence
		unless (-d "$home->{SetRoot}$dir")
		{
			# Create and verify
			# Successful
			if (mkdir("$home->{SetRoot}$dir", 0755))
			{
				# Retreive header and readme
				($header) = $database->DocumentGetHomeHeader();
				($readme) = $database->DocumentGetHomeReadme();

				# Write contents
				FileWrite("$home->{SetRoot}$dir/$home->{SetHEADER}", $header);
				FileWrite("$home->{SetRoot}$dir/$home->{SetREADME}", $readme);

				# Update Database
				$data->do("UPDATE Puppeteer 
					SET   PuppeteerHome=? 
					WHERE PuppeteerLogin=?", {}, 
				 $dir,
				 $param->{'USER'});
				
				# Return True
				return 1;
			} 

			# Fail
			else
			{
				# Display error message
				$table->MakeTop();
				$table->MakeSingle($home->{'MsgFail'});
				$table->MakeBlank();
				$table->MakeSingle("<CENTER>$!</CENTER>");
				$table->MakeBottom();
			}
		}
		else
		{ DirHandler($database, $cgi, $param, $table, $tmpl, $home, $home->{'MsgExists'}); }
		
		# Return false
		return 0; 
	} 

	# New page setup
 	else	
	{
		# Start form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}), "\n";

		# Display creation form
		$table->MakeTop();

		# Message display
		if (defined($message)) 
		{
			$table->MakeSingle($message); 
			$table->MakeBlank($cgi);
		}

		$table->MakeSingle($tmpl->PassTitle($home->{'TagHome'}));
		$table->MakeSingle($cgi->textfield('DCREATE', '', 55, 55));
		$table->MakeBlank();
		$table->MakeValid('&nbsp;', $cgi->submit($home->{'TxtCreate'}));
		$table->MakeBottom();

		# Embed and close form
		print $cgi->end_form(), "\n";

		# Return false
		return 0;
	}
}

#####################
# Directory Listing
#
# Used to show a user all files in the folder and allow them to select it
# if needed.  This is used if directory indexing has been disabled.

sub DirListing							# Database, CGI, Param and Home
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $home     = shift;					# Home page hash handle 

	my $bytes;						# Individual size
	my $count;						# File count
	my $dir;						# Users Directory
	my $file;						# Individual file
	my $size;						# Size of files

	my $inline;						# Inline parameters
	my $url      = $cgi->url();				# Back to main

	my @files;						# File listing
	my @listing;						# Listing of files


	#####################
	# Program Area

	# Cleanup
	$param->Cleanup('LISTING');


	# Data retreival
	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Retreive file listing
	($size, $count) = FileListing($home, "$home->{SetRoot}$dir", \@files); 

	# Generate inline
	($inline) = $param->EmbedInline($cgi, %{$param});



	# Top of table
	$table->MakeTop();

	# Back Button
	$table->MakeSingle("<CENTER><A HREF=\"$url$inline\">$home->{TagBack}</A></CENTER>");
	$table->MakeBlank();

	# Url listing
	$table->MakeSingle($home->{'TagUrl'});
	$table->MakeSingle("<A HREF=\"$home->{SetUrl}$dir\" TARGET=\"_new\">$home->{SetUrl}$dir</A>");
	$table->MakeBlank();

	# Directory listing
	$table->MakeSingle($home->{'TagListing'});

	# Cycle and list files
	foreach $file (sort(@files))
	{
		# Retreive file size
		$bytes = -s "$home->{SetRoot}$dir/$file";

		# Filename is escaped
		$file = $cgi->escape($file);

		# Display Filename
		$table->MakeSingle("<A HREF=\"$home->{SetUrl}$dir/$file\" TARGET=\"_new\">$home->{SetUrl}$dir/$file</A> ($bytes Bytes)");
	}

	# Useage Summary
	$table->MakeBlank();
	$table->MakeSingle($home->{'TagSummary'});
	$table->MakeSingle("$size $home->{TagStatusSize} $count $home->{TagStatusCount}");
	$table->MakeBottom();
}

#####################
# File listing 
#
# Handles the recovery of file information from within a users personal home 
# folder.   This will return size and file count for convenience.

sub FileListing 						# Hash, Directory name, Array Ref 
{
	#####################
	# Data members
	my $home     = shift;					# Home hash
	my $dir      = shift;					# Directory name 
	my $files    = shift;					# Array of files

	my $file;						# File name
 
	my $count = 0;						# File count
	my $size  = 0;						# File size


	#####################
	# Program area

	# Open Directory Handle 
 	opendir(DIR, $dir) or return $size, $count;	

	# Read through directory listing
	while ($file = readdir(DIR))
	{
		# Possible skips
		next if ($file =~ /^\./);
		next if ($file eq $home->{'SetHEADER'});
		next if ($file eq $home->{'SetREADME'});			

		# Add to size and increment
		$size += -s "$dir/$file";
		$count++;

		# Append to array
		push(@{$files}, $file); 
	}	

	# Close handle
	closedir(DIR);
	
	# Return values
	return $size, $count;
}


#####################
# File Write 
#
# As the name implies, this subroutine is designed to hide some of the more
# complex features of file writing.  This system will write whatever was
# received in the parameter list.   Overwriting existing data. 

sub FileWrite 	 					 	# File name, contents	
{
	#####################
	# Data members
	my $file     = shift;					# Name of file
	my $contents = shift;					# Contents of file

	#####################
	# Program area

	# Open file
	if (open(FILE, ">$file"))
	{
		# Write out contents
		print FILE $contents;

		# Close handle
		close(FILE);
	}

	# Return error
	else
	{ return $!; }
}


#####################
# HomeHandler 
#
# Provides the interface to see all files within their home folders as well
# as allow for additions and removals as one sees fit.  

sub HomeHandler  						# Database, CGI, Parameter list, Board hash
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handler
	my $tmpl     = shift;					# Template handler
	my $home     = shift;					# Home page hash handle 
	my $upload   = shift;					# File upload

	my $button;						# Buttons used
	my $count;						# File listing
	my $dir;						# Directory
	my $size;						# File size
	my $sorted;						# Sorted file CGI

	my $contents;						# File contents
	my $file;						# File name
	my $type;						# File type

	my $message;						# Error message

	my $inline;						# Inline parameters
	my $url      = $cgi->url();				# Back to main

	my @files;						# Files list
	my @sorted;						# Sorted files

	#####################
	# Program area

	# Retreive directory handle 
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Operations
	# Upload field handle
	if (defined($upload))
	{
		# Retreive MIME Type
		$type = $cgi->uploadInfo($param->{'FUPLOAD'})->{'Content-Type'};

		# Filename handling
		$file = $param->{'FUPLOAD'}; 
		$file =~ m!([^/:\\]*)$!;
		$file = $1;  

		# Only allow specific MIME types
		if (($type =~ /^image/i) || ($type =~ /^text/i))
		{
			unless (($file =~ /^\./) || ($file eq $home->{'SetHEADER'}) || ($file eq $home->{'SetREADME'}))
			{
				# Retreive information
				while (<$upload>) { $contents = "$contents$_"; }
			
				# Write information
				FileWrite("$home->{SetRoot}$dir/$file", $contents); 

				# Message
				$message = $home->{'MsgCreate'};
			}
			# Otherwise error message
			else
			{ $message = $home->{'MsgFail'}; }
		}
		# Otherwise error message
		else
		{ $message = $home->{'MsgFail'}; }

		# Cleanup
		$param->Cleanup(
			'FUPLOAD',
			$home->{'TxtCreate'}
		 );
	}

	# Remove files
	if ((defined($param->{$home->{'TxtDelete'}}))
	 && ($param->{'FSELECT'} ne $home->{'TxtNone'})
	 && (defined($param->{'CONFIRM'})))
	{
		# Remove file
		unlink("$home->{SetRoot}$dir/$param->{'FSELECT'}");

		# Assign message
		$message = $home->{'MsgDelete'};

		# Cleanup
		$param->Cleanup(
			{'FSELECT'},
			$home->{'TxtDelete'}
		 );	
	}

	# Always clean
	$param->Cleanup('CONFIRM');


	# Data handling
	# Retreive file listing
	($size, $count) = FileListing($home, "$home->{SetRoot}$dir", \@files); 

	# Sort and prepare 
	@sorted = sort(@files);
	unshift(@sorted, $home->{'TxtNone'});

	$sorted = $cgi->scrolling_list('FSELECT',\@sorted,$sorted[0],15);

	# Generate inline
	($inline) = $param->EmbedInline($cgi, %{$param}, 'LISTING', 'True');


	# Generate widget
	$button = $cgi->submit($home->{'TxtDelete'}) . " &nbsp; " . $cgi->checkbox(-name=>'CONFIRM', -label=>'');


	# HTML Display
	# Start form
	print $cgi->start_multipart_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Table display
	$table->MakeTop();

	# Message
	if (defined($message)) 
	{ 
		$table->MakeSingle($message); 
		$table->MakeBlank(); 
	}

	# File listing
	$table->MakeSingle($tmpl->PassTitle($home->{'TagListing'}));
	$table->MakeValid($sorted, $button, 'middle');

	# Addition of files
	if ($size < ($home->{'SetSize'} * 1048576))
	{
		# Display addition area
		$table->MakeBlank();
		$table->MakeSingle($tmpl->PassTitle($home->{'TagAdd'}));
		$table->MakeSingle($cgi->filefield('FUPLOAD', '', 40, 80));
		$table->MakeSingle($cgi->submit($home->{'TxtCreate'}));
	}
	# Display lack of capability
	else { $table->MakeSingle($home->{'MsgSpace'}); }

	# Url listing
	$table->MakeBlank();
	$table->MakeSingle($tmpl->PassTitle($home->{'TagUrl'}));
	$table->MakeSingle("<A HREF=\"$home->{SetUrl}$dir\" TARGET=\"blank\">$home->{SetUrl}$dir</A>");
	
	# Advanced listing
	$table->MakeBlank();
	$table->MakeSingle("<A HREF=\"$url$inline\">$home->{TagListingAdv}</A>");


	# Useage Summary
	$table->MakeBlank();
	$table->MakeSingle($tmpl->PassTitle($home->{'TagSummary'}));
	$table->MakeSingle("$size $home->{TagStatusSize} $count $home->{TagStatusCount}");
	$table->MakeBottom();

	# Embed and close form
	print $cgi->end_form(), "\n";
}
